@@ -1,15 +1,18 @@
-use super::consts::NOT;
+use super::consts::{LAZY, NOT};
 use super::enums::*;
 use super::error_messages::ErrorMessage;
 use super::ident_parser::{IdentParser, Rule};
 use super::utils::{
     alphabetic_first_char, first_inner, first_last_inner_str, last_inner, map_results, nth_inner,
-    symbol_variants, to_char, unquote_escape_literal, unquote_escape_raw,
+    to_char, unquote_escape_literal, unquote_escape_raw,
 };
 use crate::errors::ParseError;
 use pest::{iterators::Pair, Parser};
+use std::collections::HashMap;
 
 pub fn to_ast(source: &str) -> Result<Vec<Node>, ParseError> {
+    let mut variables: HashMap<String, Vec<Node>> = HashMap::new();
+
     let mut pairs = IdentParser::parse(Rule::root, source).map_err(|error| ParseError {
         message: error.to_string(),
     })?;
@@ -21,14 +24,17 @@ pub fn to_ast(source: &str) -> Result<Vec<Node>, ParseError> {
         .ok_or_else(|| ParseError::from(ErrorMessage::MissingRootNode))?;
 
     for statement in root.into_inner() {
-        let node = create_ast_node(statement)?;
+        let node = create_ast_node(statement, &mut variables)?;
         ast.push(node);
     }
 
     Ok(ast)
 }
 
-fn create_ast_node(pair: Pair<Rule>) -> Result<Node, ParseError> {
+fn create_ast_node(
+    pair: Pair<Rule>,
+    variables: &mut HashMap<String, Vec<Node>>,
+) -> Result<Node, ParseError> {
     Ok(match pair.as_rule() {
         Rule::raw => Node::Atom(unquote_escape_raw(&pair)),
         Rule::literal => Node::Atom(unquote_escape_literal(&pair)),
@@ -38,44 +44,75 @@ fn create_ast_node(pair: Pair<Rule>) -> Result<Node, ParseError> {
 
             let negative = negative == NOT;
 
+            if negative {
+                match ident {
+                    "start" => return Err(ErrorMessage::NegativeStartNotAllowed.into()),
+                    "end" => return Err(ErrorMessage::NegativeEndNotAllowed.into()),
+                    _ => {}
+                }
+            }
+
             match ident {
-                "space" => Node::Symbol(symbol_variants(negative, false, Symbol::Space, None)?),
-                "newline" => Node::Symbol(symbol_variants(negative, false, Symbol::Newline, None)?),
-                "vertical" => {
-                    Node::Symbol(symbol_variants(negative, false, Symbol::Vertical, None)?)
-                }
-                "word" => Node::Symbol(symbol_variants(
+                "space" => Node::Symbol(Symbol {
+                    kind: SymbolKind::Space,
                     negative,
-                    true,
-                    Symbol::Word,
-                    Some(Symbol::NotWord),
-                )?),
-                "digit" => Node::Symbol(symbol_variants(
+                }),
+                "newline" => Node::Symbol(Symbol {
+                    kind: SymbolKind::Newline,
                     negative,
-                    true,
-                    Symbol::Digit,
-                    Some(Symbol::NotDigit),
-                )?),
-                "whitespace" => Node::Symbol(symbol_variants(
+                }),
+                "vertical" => Node::Symbol(Symbol {
+                    kind: SymbolKind::Vertical,
                     negative,
-                    true,
-                    Symbol::Whitespace,
-                    Some(Symbol::NotWhitespace),
-                )?),
-                "return" => Node::Symbol(symbol_variants(negative, false, Symbol::Return, None)?),
-                "tab" => Node::Symbol(symbol_variants(negative, false, Symbol::Tab, None)?),
-                "null" => Node::Symbol(symbol_variants(negative, false, Symbol::Null, None)?),
-                "alphabet" => {
-                    Node::Symbol(symbol_variants(negative, false, Symbol::Alphabet, None)?)
-                }
-                "feed" => Node::Symbol(symbol_variants(negative, false, Symbol::Feed, None)?),
-                "char" => Node::Symbol(symbol_variants(negative, false, Symbol::Char, None)?),
-                "backspace" => {
-                    Node::Symbol(symbol_variants(negative, false, Symbol::Backspace, None)?)
-                }
-                "boundary" => {
-                    Node::Symbol(symbol_variants(negative, false, Symbol::Boundary, None)?)
-                }
+                }),
+                "word" => Node::Symbol(Symbol {
+                    kind: SymbolKind::Word,
+                    negative,
+                }),
+                "digit" => Node::Symbol(Symbol {
+                    kind: SymbolKind::Digit,
+                    negative,
+                }),
+                "whitespace" => Node::Symbol(Symbol {
+                    kind: SymbolKind::Whitespace,
+                    negative,
+                }),
+                "boundary" => Node::Symbol(Symbol {
+                    kind: SymbolKind::Boundary,
+                    negative,
+                }),
+                "alphabetic" => Node::Symbol(Symbol {
+                    kind: SymbolKind::Alphabetic,
+                    negative,
+                }),
+                "alphanumeric" => Node::Symbol(Symbol {
+                    kind: SymbolKind::Alphanumeric,
+                    negative,
+                }),
+                "return" => Node::Symbol(Symbol {
+                    kind: SymbolKind::Return,
+                    negative,
+                }),
+                "tab" => Node::Symbol(Symbol {
+                    kind: SymbolKind::Tab,
+                    negative,
+                }),
+                "null" => Node::Symbol(Symbol {
+                    kind: SymbolKind::Null,
+                    negative,
+                }),
+                "feed" => Node::Symbol(Symbol {
+                    kind: SymbolKind::Feed,
+                    negative,
+                }),
+                "char" => Node::Symbol(Symbol {
+                    kind: SymbolKind::Char,
+                    negative,
+                }),
+                "backspace" => Node::Symbol(Symbol {
+                    kind: SymbolKind::Backspace,
+                    negative,
+                }),
 
                 "start" => Node::SpecialSymbol(SpecialSymbol::Start),
                 "end" => Node::SpecialSymbol(SpecialSymbol::End),
@@ -109,8 +146,8 @@ fn create_ast_node(pair: Pair<Rule>) -> Result<Node, ParseError> {
 
         Rule::quantifier => {
             let quantity = first_inner(pair.clone());
-            let kind = first_inner(quantity);
-            let expression = create_ast_node(last_inner(pair))?;
+            let kind = first_inner(quantity.clone());
+            let expression = create_ast_node(last_inner(pair), variables)?;
 
             let expression = match expression {
                 Node::Group(group) => Expression::Group(group),
@@ -129,30 +166,38 @@ fn create_ast_node(pair: Pair<Rule>) -> Result<Node, ParseError> {
                 Node::Assertion(_) => {
                     return Err(ErrorMessage::UnexpectedAssertionInQuantifier.into())
                 }
-                Node::EndOfInput => {
-                    return Err(ErrorMessage::UnexpectedEndOfInputInQuantifier.into())
+                Node::Empty => return Err(ErrorMessage::UnexpectedEmptyNodeInQuantifier.into()),
+                Node::VariableInvocation(_) => {
+                    return Err(ErrorMessage::UnexpectedVariableInvocationInQuantifier.into())
                 }
             };
 
+            let lazy = quantity.as_str().starts_with(LAZY);
+
             match kind.as_rule() {
                 Rule::amount => Node::Quantifier(Quantifier {
                     kind: QuantifierKind::Amount(kind.as_str().to_owned()),
+                    lazy,
                     expression: Box::new(expression),
                 }),
                 Rule::over => Node::Quantifier(Quantifier {
                     kind: QuantifierKind::Over(last_inner(kind).as_str().to_owned()),
+                    lazy,
                     expression: Box::new(expression),
                 }),
                 Rule::option => Node::Quantifier(Quantifier {
                     kind: QuantifierKind::Option,
+                    lazy,
                     expression: Box::new(expression),
                 }),
                 Rule::any => Node::Quantifier(Quantifier {
                     kind: QuantifierKind::Any,
+                    lazy,
                     expression: Box::new(expression),
                 }),
                 Rule::some => Node::Quantifier(Quantifier {
                     kind: QuantifierKind::Some,
+                    lazy,
                     expression: Box::new(expression),
                 }),
 
@@ -163,6 +208,7 @@ fn create_ast_node(pair: Pair<Rule>) -> Result<Node, ParseError> {
                             start: start.to_owned(),
                             end: end.to_owned(),
                         },
+                        lazy,
                         expression: Box::new(expression),
                     })
                 }
@@ -191,7 +237,9 @@ fn create_ast_node(pair: Pair<Rule>) -> Result<Node, ParseError> {
             }
             let block = last_inner(pair);
 
-            let statements = map_results(block.into_inner(), create_ast_node)?;
+            let statements = map_results(block.into_inner(), &mut |statement| {
+                create_ast_node(statement, variables)
+            })?;
 
             Node::Group(Group {
                 ident,
@@ -215,7 +263,9 @@ fn create_ast_node(pair: Pair<Rule>) -> Result<Node, ParseError> {
 
             let block = last_inner(pair);
 
-            let statements = map_results(block.into_inner(), create_ast_node)?;
+            let statements = map_results(block.into_inner(), &mut |statement| {
+                create_ast_node(statement, variables)
+            })?;
 
             Node::Assertion(Assertion {
                 kind,
@@ -227,7 +277,28 @@ fn create_ast_node(pair: Pair<Rule>) -> Result<Node, ParseError> {
             let class = last_inner(pair.clone());
             Node::NegativeCharClass(class.as_str().to_owned())
         }
-        Rule::EOI => Node::EndOfInput,
+        Rule::variable_invocation => {
+            let identifier = last_inner(pair.clone());
+            let statements = variables.get(identifier.as_str());
+            if statements.is_none() {
+                return Err(ErrorMessage::UninitializedVariable.into());
+            }
+            Node::VariableInvocation(VariableInvocation {
+                statements: statements.unwrap().clone(),
+            })
+        }
+        Rule::variable_declaration => {
+            let identifier = first_inner(pair.clone());
+            let statements = last_inner(pair);
+            variables.insert(
+                identifier.as_str().to_owned(),
+                map_results(statements.into_inner(), &mut |statement| {
+                    create_ast_node(statement, &mut variables.clone())
+                })?,
+            );
+            Node::Empty
+        }
+        Rule::EOI => Node::Empty,
 
         _ => unreachable!(),
     })
